/*
   ADC.CPP
*/

#include "ADCTract.h"
#include <atlbase.h>

#define ASPECT_ADC_PARAMS "Software\\Aspect\\AdcParams"
#define ASPECT_ADC_PARAMS_DESC "Software\\Aspect\\AdcParams\\Description"

CADCTract :: CADCTract( LPCTSTR name, BOOL ReadOnly)
  {
  isReadOnly=ReadOnly;
  if (ReadOnly) {
   HVDSP=CreateFile ( name, GENERIC_READ,
                      FILE_SHARE_READ,
                      (LPSECURITY_ATTRIBUTES)NULL, OPEN_EXISTING,
                      FILE_ATTRIBUTE_NORMAL, (HANDLE)NULL );
  }
  else {
    HVDSP=CreateFile ( name, GENERIC_READ+GENERIC_WRITE,
                      FILE_SHARE_READ,
                      (LPSECURITY_ATTRIBUTES)NULL, OPEN_EXISTING,
                      FILE_ATTRIBUTE_NORMAL, (HANDLE)NULL );
  }
   if (HVDSP == (HANDLE) INVALID_HANDLE_VALUE)
      throw (char *)"Can't open device";
  }

CADCTract :: ~CADCTract ( void )
  {
   CloseHandle(HVDSP);
  }

int CADCTract :: ExecCommand( DWORD Command ,
                             void * inBuf, WORD inBufSize,
                             void * outBuf, USHORT outBufSize)
  {
   DWORD BytesPost=0;
   if (!DeviceIoControl(HVDSP, Command, inBuf, inBufSize, outBuf,outBufSize,
       &BytesPost, (LPOVERLAPPED)NULL ))
     {
      return GetLastError();
     }
   return 0;
  }


int CADCTract :: GetBufSize ( int *val )
  {
   *val=0;
   return ExecCommand (cmGetBufSize,NULL,0,val,4);
  }

int CADCTract :: GetRealTime ( int *val )
  {
   *val=0;
   return ExecCommand ( cmGetRealTime, NULL,0,val,4);
  }

int CADCTract :: GetLiveTime ( int *val )
  {
   *val=0;
   return ExecCommand ( cmGetLiveTime, NULL,0,val,4 );
  }

int CADCTract :: GetPresetRealTime ( int *val )
  {
   *val=0;
   return ExecCommand ( cmGetPresetRealTime, NULL,0,val,4 );
  }

int CADCTract :: GetPresetLiveTime ( int *val )
  {
   *val=0;
   return ExecCommand ( cmGetPresetLiveTime, NULL,0,val,4 );
  }

int CADCTract ::GetParamsCount (int *val)
{
  *val=0;
   return ExecCommand (cmGetParamCount, NULL,0,val, 4);
}

int CADCTract ::GetParamsList (DWORD* list, WORD listSize)
{
  *list=0;
   return ExecCommand (cmGetParamList, NULL,0,list, 4*listSize);
}

int CADCTract ::GetParamValue (DWORD parID, DWORD* val)
{
  *val=0;
   return ExecCommand (cmGetParamValue, &parID,4,val, 4);
}

int CADCTract :: SetRealTime ( int time )
{
  if (!isReadOnly)
     return ExecCommand ( cmSetRealTime, &time,4,NULL,0 );
  else return 1;
}

int CADCTract :: SetLiveTime ( int time )
{
  if (!isReadOnly)
   return ExecCommand ( cmSetLiveTime, &time,4,NULL,0 );
  else return 1;
} 

int CADCTract :: SetPresetRealTime ( int time )
{
  if (!isReadOnly)
    return ExecCommand ( cmSetPresetRealTime, &time,4,NULL,0  );
  else return 1;
} 

int CADCTract :: SetPresetLiveTime ( int time )
{
  if (!isReadOnly)
    return ExecCommand ( cmSetPresetLiveTime, &time,4,NULL,0);
  else return 1;
} 

int CADCTract :: SetBufSize ( int size )
{ 
  if (!isReadOnly)
    return ExecCommand (cmSetBufSize,&size,4,NULL,0);
  else return 1;
} 

int  CADCTract :: SetParamValue (DWORD parId, DWORD val)
{
  if (!isReadOnly) {
    tSetParamStruct param;
    param.ParamCode = parId;
    param.ParamValue= val;
    return ExecCommand (cmSetParamValue, &param,sizeof(tSetParamStruct),NULL, 0);
  }
  else return 1;
}

int CADCTract :: ClearBuf ( void )
{ 
  if (!isReadOnly) 
    return ExecCommand ( cmClearBuf,NULL,0,NULL,0 );
  else return 1;
} 


int CADCTract :: RunChannel ( void )
{ 
  if (!isReadOnly)
    return ExecCommand ( cmRunChannel, NULL,0,NULL,0 );
  else return 1;
} 

int CADCTract :: StopChannel ( void )
{ 
  if (!isReadOnly)
    return ExecCommand ( cmStopChannel,NULL,0,NULL,0 );
  else return 1;
} 

int CADCTract :: WriteBuf ( void * p,  int offset, int chCount)
{
  if (!isReadOnly) {
    USHORT offs = (USHORT)(offset & 0xFFFF);
    return ExecCommand (cmWriteBuf, &offs,2,p,chCount*4 );
  }
  else return 1;
}

int CADCTract :: ReadBuf ( void * p, int offset, int chCount)
{ 
  USHORT offs = (USHORT)(offset & 0xFFFF);
  return ExecCommand ( cmReadBuf, &offs,2,p, chCount*4);
} 

//---------------------------------------------------------------------------

int CADCTract::GetParamName(DWORD parID, char *sName, DWORD* bufLen)
{
  CRegKey regKey;
	long lRes;
	DWORD lSize;
  char valName [10], parName[256];

	lRes = regKey.Open(HKEY_LOCAL_MACHINE, ASPECT_ADC_PARAMS);
  if (ERROR_SUCCESS != lRes)
    return lRes;
  sprintf (valName,"%01u", parID);
  lSize=255;
  if (ERROR_SUCCESS == regKey.QueryValue(parName,valName , &lSize)) {
    lSize = (lSize > *bufLen) ? *bufLen : lSize;
    memcpy (sName, parName,lSize);
    *bufLen = lSize;
    return 0;
  }
  else {
    *bufLen=0;
    return 1;
  }
}


int CADCTract::GetParamDescription(DWORD parID, char *sDesc, DWORD* bufLen)
{
  CRegKey regKey;
	long lRes;
	DWORD lSize;
  char parName[256];
  char *parDesc;

  lSize=sizeof (parName);
  if (GetParamName(parID, parName,&lSize))
    return 1;

	lRes = regKey.Open(HKEY_LOCAL_MACHINE, ASPECT_ADC_PARAMS_DESC);
  if (ERROR_SUCCESS != lRes)
    return lRes;

  lSize = *bufLen;
  parDesc = new char[1000];
  if (!parDesc)
    return 1;
  if (ERROR_SUCCESS == regKey.QueryValue(parDesc,parName,&lSize)) {
    lSize = (lSize > *bufLen) ? *bufLen : lSize;
    memcpy (sDesc, parDesc,lSize);
    *bufLen = lSize;
    delete parDesc;
    return 0;
  }
  else {
    *bufLen=0;
    delete parDesc;
    return 1;
  }
}

int CADCTract::GetStatus(DWORD *status)
{
  *status=0;
  return ExecCommand (cmGetStatus, NULL,0,status,4 );
}
